"""Fix slot value index on PostgreSQL

Reason:
In migration 6738be716c3f, we created an index for the `slot_value` column of
`conversation_event`. A problem was reported where the migration failed on
PostgreSQL because some SlotSet events had very large values, and the default
index type (B-Tree) could not be applied to them. The migration was modified so
that this index was no longer created. This migration was then added, which 1)
deletes the index in case it exists (this will be the case for users which were
able to run the migration successfully) and 2) creates the new index, using GIN
instead when running on PostgreSQL.

Revision ID: 3fbc8790762e
Revises: 304e0754a200

"""
from alembic import op
import sqlalchemy as sa

import rasax.community.database.schema_migrations.alembic.utils as migration_utils

# revision identifiers, used by Alembic.
revision = "3fbc8790762e"
down_revision = "304e0754a200"
branch_labels = None
depends_on = None

TABLE_NAME = "conversation_event"
INDEX_NAME = "conversation_slot_value_index"
COLUMN = "slot_value"


def upgrade():
    # Remove the old version of the index, only if it's there
    if migration_utils.index_exists(TABLE_NAME, INDEX_NAME):
        with op.batch_alter_table(TABLE_NAME) as batch_op:
            batch_op.drop_index(INDEX_NAME)

    with op.batch_alter_table(TABLE_NAME) as batch_op:
        if migration_utils.using_dialect("postgresql"):
            batch_op.create_index(
                INDEX_NAME, [sa.text(f"tsvector({COLUMN})")], postgresql_using="gin"
            )
        else:
            batch_op.create_index(INDEX_NAME, [COLUMN])


def downgrade():
    with op.batch_alter_table(TABLE_NAME) as batch_op:
        batch_op.drop_index(INDEX_NAME)
